@@ -221,23 +221,7 @@ emit_line (GogRenderer *rend, gboolean preserve, GOPathOptions options)
 static void
 emit_fill (GogRenderer *rend, gboolean preserve)
 {
-	GOStyle const *style = rend->cur_style;
-	cairo_t *cr = rend->cairo;
-	cairo_pattern_t *cr_pattern = NULL;
-
-	cr_pattern = go_style_create_cairo_pattern (style, cr);
-	if (cr_pattern == NULL) {
-		if (!preserve)
-			cairo_new_path (cr);
-		return;
-	}
-	cairo_set_source (cr, cr_pattern);
-	cairo_pattern_destroy (cr_pattern);
-
-	if (preserve)
-		cairo_fill_preserve (cr);
-	else
-		cairo_fill (cr);
+	go_style_fill (rend->cur_style, rend->cairo, preserve);
 }
 
 static void
@@ -487,6 +471,41 @@ gog_renderer_fill_shape (GogRenderer *renderer, GOPath const *path)
 	_draw_shape (renderer, path, TRUE, FALSE);
 }
 
+static void
+_draw_rotated_shape (GogRenderer *renderer, GOPath const *path, gboolean fill, gboolean stroke, gboolean rotate_bg)
+{
+	GOStyle const *style;
+	GOPathOptions line_options;
+	double width;
+
+	g_return_if_fail (GOG_IS_RENDERER (renderer));
+	g_return_if_fail (renderer->cur_style != NULL);
+	g_return_if_fail (GO_IS_PATH (path));
+
+        style = renderer->cur_style;
+
+	line_options = go_path_get_options (path);
+	width = stroke ? _grc_line_size (renderer, style->line.width,
+					 line_options & GO_PATH_OPTIONS_SNAP_WIDTH) : 0;
+
+	cairo_save (renderer->cairo);
+	cairo_rotate (renderer->cairo, -renderer->cur_style->text_layout.angle *M_PI / 180.);
+	path_interpret (renderer, path, width);
+
+	if (fill) {
+		if (rotate_bg) {
+			emit_fill (renderer, stroke);
+			cairo_restore (renderer->cairo);
+		} else {
+			cairo_restore (renderer->cairo);
+			emit_fill (renderer, stroke);
+		}
+	}
+
+	if (stroke)
+		emit_line (renderer, FALSE, go_path_get_options (path));
+}
+
 /*****************************************************************************/
 
 /**
@@ -661,6 +680,45 @@ gog_renderer_fill_rectangle (GogRenderer *rend, GogViewAllocation const *rect)
 	_draw_rectangle (rend, rect, TRUE, FALSE);
 }
 
+static void
+_draw_rotated_rectangle (GogRenderer *rend, GogViewAllocation const *rect, gboolean fill, gboolean stroke, gboolean rotate_bg)
+{
+	GOStyle const *style;
+	GOPath *path;
+	gboolean narrow = (rect->w < 3.) || (rect->h < 3.);
+	double o, o_2;
+
+	g_return_if_fail (GOG_IS_RENDERER (rend));
+	g_return_if_fail (GO_IS_STYLE (rend->cur_style));
+
+	style = rend->cur_style;
+	narrow |= !go_style_is_outline_visible (style);
+
+	path = go_path_new ();
+	go_path_set_options (path, GO_PATH_OPTIONS_SHARP);
+
+	if (!narrow) {
+		o = gog_renderer_line_size (rend, style->line.width);
+		o_2 = o / 2.;
+	} else
+		o = o_2 = 0.;
+
+	go_path_rectangle (path, 0., 0., rect->w - o, rect->h - o);
+
+	cairo_save (rend->cairo);
+	cairo_translate (rend->cairo, rect->x - o_2, rect->y - o_2);
+	_draw_rotated_shape (rend, path, fill, stroke && !narrow, rotate_bg);
+	cairo_restore (rend->cairo);
+
+	go_path_free (path);
+}
+
+void
+gog_renderer_draw_rotated_rectangle (GogRenderer *rend, GogViewAllocation const *rect, gboolean rotate_bg)
+{
+	_draw_rotated_rectangle (rend, rect, TRUE, TRUE, rotate_bg);
+}
+
 /**
  * gog_renderer_draw_grip:
  * @renderer : #GogRenderer
@@ -776,6 +834,91 @@ gog_renderer_draw_marker (GogRenderer *rend, double x, double y)
 	cairo_paint (rend->cairo);
 }
 
+/**
+ * gog_renderer_draw_gostring :
+ * @rend   : #GogRenderer
+ * @gostring : the #GOString to draw
+ * @pos    : #GogViewAllocation
+ * @anchor : #GOAnchorType how to draw relative to @pos
+ *
+ * Have @rend draw @layout in the at @pos.{x,y} anchored by the @anchor corner.
+ * If @pos.w or @pos.h are >= 0 then clip the results to less than that size.
+ **/
+
+void
+gog_renderer_draw_gostring (GogRenderer *rend, GOString *str,
+			GogViewAllocation const *pos, GOAnchorType anchor)
+{
+	PangoLayout *layout;
+	PangoContext *context;
+	cairo_t *cairo;
+	GOGeometryOBR obr;
+	GOGeometryAABR aabr;
+	GOStyle const *style;
+	int iw, ih;
+	PangoAttrList *attr;
+
+	g_return_if_fail (str != NULL);
+	g_return_if_fail (GOG_IS_RENDERER (rend));
+	g_return_if_fail (rend->cur_style != NULL);
+
+	cairo = rend->cairo;
+	style = rend->cur_style;
+
+	/* Note: orig layout may not have been created using cairo! */
+	layout = pango_cairo_create_layout (cairo);
+	context = pango_layout_get_context (layout);
+	pango_layout_set_text (layout, str->str, -1);
+	attr = go_string_get_markup (str);
+	if (attr)
+		pango_layout_set_attributes (layout, attr);
+	pango_cairo_context_set_resolution (context, 72.0);
+	pango_layout_set_font_description (layout, style->font.font->desc);
+	pango_layout_get_size (layout, &iw, &ih);
+
+	obr.w = rend->scale * ((double) iw + (double) PANGO_SCALE / 2.0)
+		/ (double) PANGO_SCALE;
+	obr.h = rend->scale * ((double) ih + (double) PANGO_SCALE / 2.0)
+		/(double) PANGO_SCALE;
+	obr.alpha = -style->text_layout.angle * M_PI / 180.0;
+	obr.x = pos->x;
+	obr.y = pos->y;
+	go_geometry_OBR_to_AABR (&obr, &aabr);
+
+	switch (anchor) {
+		case GO_ANCHOR_NW: case GO_ANCHOR_W: case GO_ANCHOR_SW:
+			obr.x += aabr.w / 2.0;
+			break;
+		case GO_ANCHOR_NE : case GO_ANCHOR_SE : case GO_ANCHOR_E :
+			obr.x -= aabr.w / 2.0;
+			break;
+		default : break;
+	}
+
+	switch (anchor) {
+		case GO_ANCHOR_NW: case GO_ANCHOR_N: case GO_ANCHOR_NE:
+			obr.y += aabr.h / 2.0;
+			break;
+		case GO_ANCHOR_SE : case GO_ANCHOR_S : case GO_ANCHOR_SW :
+			obr.y -= aabr.h / 2.0;
+			break;
+		default : break;
+	}
+
+	cairo_save (cairo);
+	cairo_set_source_rgba (cairo, GO_COLOR_TO_CAIRO (style->font.color));
+	cairo_move_to (cairo, obr.x - (obr.w / 2.0) * cos (obr.alpha) +
+		       (obr.h / 2.0) * sin (obr.alpha),
+		       obr.y - (obr.w / 2.0) * sin (obr.alpha) -
+		       (obr.h / 2.0) * cos (obr.alpha));
+	cairo_rotate (cairo, obr.alpha);
+	cairo_scale (cairo, rend->scale, rend->scale);
+	pango_cairo_show_layout (cairo, layout);
+	cairo_restore (cairo);
+	g_object_unref (layout);
+}
+
+
 /**
  * gog_renderer_draw_text :
  * @rend   : #GogRenderer
@@ -790,74 +933,85 @@ gog_renderer_draw_marker (GogRenderer *rend, double x, double y)
 
 void
 gog_renderer_draw_text (GogRenderer *rend, char const *text,
-			GogViewAllocation const *pos, GtkAnchorType anchor,
+			GogViewAllocation const *pos, GOAnchorType anchor,
 			gboolean use_markup)
 {
-	PangoLayout *layout;
-	PangoContext *context;
-	cairo_t *cairo = rend->cairo;
-	GOGeometryOBR obr;
-	GOGeometryAABR aabr;
-	GOStyle const *style;
-	int iw, ih;
+	GOString *str;
+	PangoAttrList *attr_list = NULL;
+	char *m_text = NULL;
 
 	g_return_if_fail (GOG_IS_RENDERER (rend));
-	g_return_if_fail (rend->cur_style != NULL);
 	g_return_if_fail (text != NULL);
 
 	if (*text == '\0')
 		return;
 
-	style = rend->cur_style;
+	if (use_markup && pango_parse_markup  (text, -1, 0,
+					       &attr_list, &m_text,
+					       NULL, NULL))
+		str = go_string_new_rich_nocopy (m_text, -1, attr_list, NULL);
+	else
+		str = go_string_new (text);
+	gog_renderer_draw_gostring (rend, str, pos, anchor);
+	go_string_unref (str);
+}
 
+/**
+ * gog_renderer_get_text_OBR :
+ * @rend: #GogRenderer
+ * @gostring: the string to draw
+ * @obr: #GOGeometryOBR to store the Object Bounding Rectangle of @text.
+ **/
+void
+gog_renderer_get_gostring_OBR (GogRenderer *rend, GOString *str, GOGeometryOBR *obr)
+{
+	GOStyle const *style;
+	PangoLayout *layout;
+	PangoContext *context;
+	PangoRectangle logical;
+	cairo_t *cairo;
+	PangoAttrList *attr;
+
+	g_return_if_fail (GOG_IS_RENDERER (rend));
+	g_return_if_fail (rend->cur_style != NULL);
+	g_return_if_fail (obr != NULL);
+
+	cairo = rend->cairo;
+	obr->x = obr->y = 0;
+	if (str->str == NULL || *(str->str) == '\0') {
+		/* Make sure invisible things don't skew size */
+		obr->w = obr->h = 0;
+		obr->alpha = 0;
+		return;
+	}
+
+	style = rend->cur_style;
 	layout = pango_cairo_create_layout (cairo);
 	context = pango_layout_get_context (layout);
+	pango_layout_set_text (layout, str->str, -1);
+	attr = go_string_get_markup (str);
+	if (attr)
+		pango_layout_set_attributes (layout, attr);
 	pango_cairo_context_set_resolution (context, 72.0);
-	if (use_markup)
-		pango_layout_set_markup (layout, text, -1);
-	else
-		pango_layout_set_text (layout, text, -1);
 	pango_layout_set_font_description (layout, style->font.font->desc);
-	pango_layout_get_size (layout, &iw, &ih);
-
-	obr.w = rend->scale * ((double) iw + (double) PANGO_SCALE / 2.0) / (double) PANGO_SCALE;
-	obr.h = rend->scale * ((double) ih + (double) PANGO_SCALE / 2.0) /(double) PANGO_SCALE;
-	obr.alpha = -style->text_layout.angle * M_PI / 180.0;
-	obr.x = pos->x;
-	obr.y = pos->y;
-	go_geometry_OBR_to_AABR (&obr, &aabr);
-
-	switch (anchor) {
-		case GTK_ANCHOR_NW: case GTK_ANCHOR_W: case GTK_ANCHOR_SW:
-			obr.x += aabr.w / 2.0;
-			break;
-		case GTK_ANCHOR_NE : case GTK_ANCHOR_SE : case GTK_ANCHOR_E :
-			obr.x -= aabr.w / 2.0;
-			break;
-		default : break;
-	}
-
-	switch (anchor) {
-		case GTK_ANCHOR_NW: case GTK_ANCHOR_N: case GTK_ANCHOR_NE:
-			obr.y += aabr.h / 2.0;
-			break;
-		case GTK_ANCHOR_SE : case GTK_ANCHOR_S : case GTK_ANCHOR_SW :
-			obr.y -= aabr.h / 2.0;
-			break;
-		default : break;
-	}
-
-	cairo_save (cairo);
-	cairo_set_source_rgba (cairo, GO_COLOR_TO_CAIRO (style->font.color));
-	cairo_move_to (cairo, obr.x - (obr.w / 2.0) * cos (obr.alpha) + (obr.h / 2.0) * sin (obr.alpha),
-		       obr.y - (obr.w / 2.0) * sin (obr.alpha) - (obr.h / 2.0) * cos (obr.alpha));
-	cairo_rotate (cairo, obr.alpha);
-	cairo_scale (cairo, rend->scale, rend->scale);
-	pango_cairo_show_layout (cairo, layout);
-	cairo_restore (cairo);
+	pango_layout_get_extents (layout, NULL, &logical);
 	g_object_unref (layout);
+
+	obr->w = rend->scale * ((double) logical.width + (double) PANGO_SCALE / 2.0)
+		/ (double) PANGO_SCALE;
+	obr->h = rend->scale * ((double) logical.height + (double) PANGO_SCALE / 2.0)
+		/(double) PANGO_SCALE;
+
+	/* Make sure invisible things don't skew size */
+	if (obr->w == 0)
+		obr->h = 0;
+	else if (obr->h == 0)
+		obr->w = 0;
+
+	obr->alpha = - style->text_layout.angle * M_PI / 180.0;
 }
 
+
 /**
  * gog_renderer_get_text_OBR :
  * @rend: #GogRenderer
@@ -869,55 +1023,29 @@ void
 gog_renderer_get_text_OBR (GogRenderer *rend, char const *text,
 			   gboolean use_markup, GOGeometryOBR *obr)
 {
-	GOStyle const *style;
-	PangoLayout *layout;
-	PangoContext *context;
-	PangoRectangle logical;
-	cairo_t *cairo = rend->cairo;
+	GOString *str;
+	PangoAttrList *attr_list = NULL;
+	char *m_text = NULL;
 
 	g_return_if_fail (GOG_IS_RENDERER (rend));
-	g_return_if_fail (rend->cur_style != NULL);
 	g_return_if_fail (text != NULL);
-	g_return_if_fail (obr != NULL);
 
-	obr->x = obr->y = 0;
-	if (*text == '\0') {
-		/* Make sure invisible things don't skew size */
-		obr->w = obr->h = 0;
-		obr->alpha = 0;
-		return;
-	}
-
-	style = rend->cur_style;
-
-	layout = pango_cairo_create_layout (cairo);
-	context = pango_layout_get_context (layout);
-	pango_cairo_context_set_resolution (context, 72.0);
-	if (use_markup)
-		pango_layout_set_markup (layout, text, -1);
+	if (use_markup && pango_parse_markup  (text, -1, 0,
+					       &attr_list, &m_text,
+					       NULL, NULL))
+		str = go_string_new_rich_nocopy (m_text, -1, attr_list, NULL);
 	else
-		pango_layout_set_text (layout, text, -1);
-	pango_layout_set_font_description (layout, style->font.font->desc);
-	pango_layout_get_extents (layout, NULL, &logical);
-	g_object_unref (layout);
+		str = go_string_new (text);
+	gog_renderer_get_gostring_OBR (rend, str, obr);
+	go_string_unref (str);
 
-	obr->w = rend->scale * ((double) logical.width + (double) PANGO_SCALE / 2.0) / (double) PANGO_SCALE;
-	obr->h = rend->scale * ((double) logical.height + (double) PANGO_SCALE / 2.0) /(double) PANGO_SCALE;
-
-	/* Make sure invisible things don't skew size */
-	if (obr->w == 0)
-		obr->h = 0;
-	else if (obr->h == 0)
-		obr->w = 0;
-
-	obr->alpha = - style->text_layout.angle * M_PI / 180.0;
 }
 
 /**
  * gog_renderer_get_text_AABR :
  * @rend: #GogRenderer
  * @text: the string to draw
- * @use_markup: wether to use pango markup
+ * @use_markup: whether to use pango markup
  * @aabr: #GOGeometryAABR to store the Axis Aligned Bounding Rectangle of @text.
  **/
 void
@@ -930,6 +1058,170 @@ gog_renderer_get_text_AABR (GogRenderer *rend, char const *text,
 	go_geometry_OBR_to_AABR (&obr, aabr);
 }
 
+/**
+ * gog_renderer_get_gostring_AABR :
+ * @rend: #GogRenderer
+ * @str: the string to draw
+ * @aabr: #GOGeometryAABR to store the Axis Aligned Bounding Rectangle of @text.
+ **/
+void
+gog_renderer_get_gostring_AABR (GogRenderer *rend, GOString *str,
+			    GOGeometryAABR *aabr)
+{
+	GOGeometryOBR obr;
+
+	gog_renderer_get_gostring_OBR (rend, str, &obr);
+	go_geometry_OBR_to_AABR (&obr, aabr);
+}
+
+void
+gog_renderer_draw_data_label (GogRenderer *rend, GogSeriesLabelElt const *elt,
+                              GogViewAllocation const *pos, GOAnchorType anchor,
+                              GOStyle *legend_style)
+{
+	/* things are a bit different from gog_renderer_draw_gostring, so the
+	 * code must be copied */
+	PangoLayout *layout;
+	PangoContext *context;
+	cairo_t *cairo;
+	GOGeometryOBR obr;
+	GOGeometryAABR aabr;
+	GOStyle const *style;
+	int iw, ih;
+	PangoAttrList *attrs;
+	PangoRectangle rect;
+
+	g_return_if_fail (elt != NULL && elt->str != NULL);
+	g_return_if_fail (GOG_IS_RENDERER (rend));
+	g_return_if_fail (rend->cur_style != NULL);
+
+	cairo = rend->cairo;
+	style = (GO_IS_STYLED_OBJECT (elt->point))?
+		go_styled_object_get_style (GO_STYLED_OBJECT (elt->point)):
+		rend->cur_style;
+
+	/* Note: orig layout may not have been created using cairo! */
+	layout = pango_cairo_create_layout (cairo);
+	pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
+	context = pango_layout_get_context (layout);
+	pango_layout_set_text (layout, elt->str->str, -1);
+	attrs = pango_attr_list_copy (go_string_get_markup (elt->str));
+	pango_layout_set_font_description (layout, style->font.font->desc);
+	if (attrs)
+		pango_layout_set_attributes (layout, attrs);
+	pango_cairo_context_set_resolution (context, 72.0);
+	/*now get the real size */
+	pango_layout_get_size (layout, &iw, &ih);
+	if (elt->legend_pos >= 0) {
+		/* we need to add enough room to draw the legend entry */
+		PangoRectangle rect;
+		PangoAttribute *attr;
+		rect.x = rect.y = 0;
+		pango_layout_get_size (layout, &iw, &ih);
+		rect.height = 1; /* only the width is important */
+		rect.width = (legend_style->interesting_fields & GO_STYLE_LINE)? 2 * ih: ih;
+		attr = pango_attr_shape_new (&rect, &rect);
+		attr->start_index = elt->legend_pos;
+		attr->end_index = elt->legend_pos + 1;
+		pango_attr_list_insert (attrs, attr);
+		pango_layout_set_attributes (layout, attrs);
+		pango_layout_get_size (layout, &iw, &ih);
+	}
+	pango_attr_list_unref (attrs);
+
+	obr.w = rend->scale * ((double) iw + (double) PANGO_SCALE / 2.0)
+		/ (double) PANGO_SCALE;
+	obr.h = rend->scale * ((double) ih + (double) PANGO_SCALE / 2.0)
+		/(double) PANGO_SCALE;
+	obr.alpha = -style->text_layout.angle * M_PI / 180.0;
+	obr.x = pos->x;
+	obr.y = pos->y;
+	go_geometry_OBR_to_AABR (&obr, &aabr);
+
+	switch (anchor) {
+		case GO_ANCHOR_NW: case GO_ANCHOR_W: case GO_ANCHOR_SW:
+			obr.x += aabr.w / 2.0;
+			break;
+		case GO_ANCHOR_NE : case GO_ANCHOR_SE : case GO_ANCHOR_E :
+			obr.x -= aabr.w / 2.0;
+			break;
+		default : break;
+	}
+
+	switch (anchor) {
+		case GO_ANCHOR_NW: case GO_ANCHOR_N: case GO_ANCHOR_NE:
+			obr.y += aabr.h / 2.0;
+			break;
+		case GO_ANCHOR_SE : case GO_ANCHOR_S : case GO_ANCHOR_SW :
+			obr.y -= aabr.h / 2.0;
+			break;
+		default : break;
+	}
+
+	cairo_save (cairo);
+	cairo_set_source_rgba (cairo, GO_COLOR_TO_CAIRO (style->font.color));
+	cairo_translate (cairo, obr.x - (obr.w / 2.0) * cos (obr.alpha) +
+		       (obr.h / 2.0) * sin (obr.alpha),
+		       obr.y - (obr.w / 2.0) * sin (obr.alpha) -
+		       (obr.h / 2.0) * cos (obr.alpha));
+	cairo_rotate (cairo, obr.alpha);
+	/* now draw the legen entry if needed */
+	if (elt->legend_pos >= 0 && legend_style != NULL) {
+		GOStyle *style = go_style_dup (legend_style);
+		GogViewAllocation rectangle;
+		double x, y,w, h;
+		pango_layout_index_to_pos (layout, elt->legend_pos, &rect);
+		x = (double) rect.x / PANGO_SCALE * rend->scale;
+		y = (double) rect.y / PANGO_SCALE * rend->scale;
+		w = (double) rect.width / PANGO_SCALE * rend->scale;
+		h = (double) rect.height / PANGO_SCALE * rend->scale;
+		if (style->interesting_fields & GO_STYLE_LINE) { /* line and marker */
+			GOPath *line_path;
+			if (style->line.width > h / 3.)
+				style->line.width = h / 3.;
+			if (go_marker_get_size (style->marker.mark) > h)
+				go_marker_set_size (style->marker.mark, h);
+			gog_renderer_push_style (rend, style);
+			line_path = go_path_new ();
+			y += h / 2.;
+			go_path_move_to (line_path, x, y);
+			go_path_line_to (line_path, x + w, y);
+			if (style->interesting_fields & GO_STYLE_FILL) {
+				rectangle.x = x;
+				rectangle.y = y;
+				rectangle.w = w;
+				rectangle.h = h / 2.0;
+				gog_renderer_fill_rectangle (rend, &rectangle);
+			}
+			gog_renderer_stroke_serie (rend, line_path);
+			go_path_free (line_path);
+			gog_renderer_draw_marker (rend, x + w / 2., y);
+		} else if (style->interesting_fields & GO_STYLE_FILL) {/* area */
+			if (style->line.width > h / 3.)
+				style->line.width = h / 3.;
+
+			rectangle.x = x;
+			rectangle.y = y;
+			rectangle.w = w;
+			rectangle.h = h;
+
+			gog_renderer_push_style (rend, style);
+			gog_renderer_draw_rectangle (rend, &rectangle);
+		} else if (style->interesting_fields & GO_STYLE_MARKER) {					/* markers only */
+			if (go_marker_get_size (style->marker.mark) > h)
+				go_marker_set_size (style->marker.mark, h);
+			gog_renderer_push_style (rend, style);
+			gog_renderer_draw_marker (rend, x + w / 2., y + h / 2.);
+		}
+		gog_renderer_pop_style (rend);
+		g_object_unref (style);
+	}
+	cairo_scale (cairo, rend->scale, rend->scale);
+	pango_cairo_show_layout (cairo, layout);
+	cairo_restore (cairo);
+	g_object_unref (layout);
+}
+
 static void
 _free_marker_data (GogRenderer *rend)
 {
@@ -1235,60 +1527,32 @@ gog_renderer_export_image (GogRenderer *rend, GOImageFormat format,
 	switch (format) {
 		case GO_IMAGE_FORMAT_EPS:
 			rend->marker_as_surface = FALSE;
-#ifdef HAVE_CAIRO_PS_SURFACE_SET_EPS
 			surface = cairo_ps_surface_create_for_stream
 				(_cairo_write_func,
 				 output, width_in_pts, height_in_pts);
 			cairo_ps_surface_set_eps (surface, TRUE);
-#ifdef HAVE_CAIRO_SURFACE_SET_FALLBACK_RESOLUTION
 			cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
-#endif
 			goto do_export_vectorial;
-#else
-			g_warning ("[GogRendererCairo::export_image] cairo EPS backend missing");
-			return FALSE;
-#endif
 		case GO_IMAGE_FORMAT_PDF:
 			rend->marker_as_surface = FALSE;
-#ifdef CAIRO_HAS_PDF_SURFACE
 			surface = cairo_pdf_surface_create_for_stream
 				(_cairo_write_func,
 				 output, width_in_pts, height_in_pts);
-#ifdef HAVE_CAIRO_SURFACE_SET_FALLBACK_RESOLUTION
 			cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
-#endif
 			goto do_export_vectorial;
-#else
-			g_warning ("[GogRendererCairo::export_image] cairo PDF backend missing");
-			return FALSE;
-#endif
 		case GO_IMAGE_FORMAT_PS:
 			rend->marker_as_surface = FALSE;
-#ifdef CAIRO_HAS_PS_SURFACE
 			surface = cairo_ps_surface_create_for_stream
 				(_cairo_write_func,
 				 output, width_in_pts, height_in_pts);
-#ifdef HAVE_CAIRO_SURFACE_SET_FALLBACK_RESOLUTION
 			cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
-#endif
 			goto do_export_vectorial;
-#else
-			g_warning ("[GogRendererCairo::export_image] cairo PS backend missing");
-			return FALSE;
-#endif
 		case GO_IMAGE_FORMAT_SVG:
 			rend->marker_as_surface = TRUE;
-#ifdef CAIRO_HAS_SVG_SURFACE
 			surface = cairo_svg_surface_create_for_stream
 				(_cairo_write_func,
 				 output, width_in_pts, height_in_pts);
-#ifdef HAVE_CAIRO_SURFACE_SET_FALLBACK_RESOLUTION
 			cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
-#endif
-#else
-			g_warning ("[GogRendererCairo::export_image] cairo SVG backend missing");
-			return FALSE;
-#endif
 do_export_vectorial:
 			rend->scale = 1.0;
 			cairo = cairo_create (surface);
@@ -1359,19 +1623,19 @@ gog_renderer_get_hairline_width_pts (GogRenderer const *rend)
 #ifdef GOFFICE_WITH_LASEM
 
 void
-gog_renderer_draw_equation (GogRenderer *renderer, LsmMathmlView *mathml_view, double x, double y)
+gog_renderer_draw_equation (GogRenderer *renderer, LsmDomView *mathml_view, double x, double y)
 {
 	cairo_t *cairo;
 	double w, h, alpha;
 
 	g_return_if_fail (GOG_IS_RENDERER (renderer));
-	g_return_if_fail (LSM_IS_MATHML_VIEW (mathml_view));
+	g_return_if_fail (LSM_IS_DOM_VIEW (mathml_view));
 	g_return_if_fail (renderer->cur_style != NULL);
 
 	cairo = renderer->cairo;
 
 	alpha = -renderer->cur_style->text_layout.angle * M_PI / 180.0;
-	lsm_dom_view_get_size (LSM_DOM_VIEW (mathml_view), &w, &h);
+	lsm_dom_view_get_size (LSM_DOM_VIEW (mathml_view), &w, &h, NULL);
 	w *= renderer->scale;
 	h *= renderer->scale;
 	x = x - (w / 2.0) * cos (alpha) + (h / 2.0) * sin (alpha);
@@ -1383,8 +1647,7 @@ gog_renderer_draw_equation (GogRenderer *renderer, LsmMathmlView *mathml_view, d
 	cairo_rotate (cairo, alpha);
 	cairo_scale (cairo, renderer->scale_x, renderer->scale_y);
 
-	lsm_dom_view_set_cairo (LSM_DOM_VIEW (mathml_view), cairo);
-	lsm_dom_view_render (LSM_DOM_VIEW (mathml_view), 0., 0.);
+	lsm_dom_view_render (LSM_DOM_VIEW (mathml_view), cairo, 0., 0.);
 
 	cairo_restore (cairo);
 }
@@ -1564,6 +1827,12 @@ gog_renderer_class_init (GogRendererClass *renderer_klass)
 		G_TYPE_NONE, 0);
 }
 
+double
+gog_renderer_get_scale (GogRenderer *renderer)
+{
+	return renderer->scale;
+}
+
 GSF_CLASS (GogRenderer, gog_renderer,
 	   gog_renderer_class_init, gog_renderer_init,
 	   G_TYPE_OBJECT)
