@@ -34,7 +34,7 @@
 #include <windows.h>
 #include <io.h>
 #endif
-#ifdef HAVE_GTK_SHOW_URI
+#ifdef GOFFICE_WITH_GTK
 #include <gtk/gtk.h>
 #endif
 #include <string.h>
@@ -502,9 +502,14 @@ go_dirname_from_uri (char const *uri, gboolean brief)
 	char *dirname_utf8, *dirname;
 	char *uri_dirname = g_path_get_dirname (uri);
 	dirname = uri_dirname ? go_filename_from_uri (uri_dirname) : NULL;
-	dirname = dirname ? g_strconcat ("file://", dirname, NULL) : NULL;
 	g_free (uri_dirname);
 
+	if (dirname) {
+		char *temp = g_strconcat ("file://", dirname, NULL);
+		g_free (dirname);
+		dirname = temp;
+	}
+
 	if (brief && dirname &&
 	    g_ascii_strncasecmp (dirname, "file:///", 8) == 0) {
 		char *temp = g_strdup (dirname + 7);
@@ -661,7 +666,7 @@ gchar *
 go_file_get_owner_name (char const *uri)
 {
 	char const *name;
-	char *nameutf8 = NULL;
+	GString *nameutf8 = NULL;
 	GFile *file = g_file_new_for_uri (uri);
 	GError *error = NULL;
 	GFileInfo *info = g_file_query_info (file,
@@ -675,8 +680,8 @@ go_file_get_owner_name (char const *uri)
 
 	name = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_OWNER_USER);
 	(void) go_guess_encoding (name, strlen (name),
-				  NULL, &nameutf8);
-	return nameutf8;
+				  NULL, &nameutf8, NULL);
+	return (nameutf8 ? g_string_free (nameutf8, FALSE) : NULL);
 }
 
 /*
@@ -687,7 +692,7 @@ gchar *
 go_file_get_group_name (char const *uri)
 {
 	char const *name;
-	char *nameutf8 = NULL;
+	GString *nameutf8 = NULL;
 	GFile *file = g_file_new_for_uri (uri);
 	GError *error = NULL;
 	GFileInfo *info = g_file_query_info (file,
@@ -701,8 +706,8 @@ go_file_get_group_name (char const *uri)
 
 	name = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_OWNER_GROUP);
 	(void) go_guess_encoding (name, strlen (name),
-				  NULL, &nameutf8);
-	return nameutf8;
+				  NULL, &nameutf8, NULL);
+	return (nameutf8 ? g_string_free (nameutf8, FALSE) : NULL);
 }
 
 GOFilePermissions *
@@ -987,7 +992,7 @@ go_url_encode (gchar const *text, int type)
 GError *
 go_url_show (gchar const *url)
 {
-#ifdef HAVE_GTK_SHOW_URI
+#ifdef GOFFICE_WITH_GTK
 	GError *error = NULL;
 	gtk_show_uri (NULL, url, GDK_CURRENT_TIME, &error);
 	return error;
@@ -1046,7 +1051,7 @@ go_url_check_extension (gchar const *uri,
  *
  * returns: the mime type for the file as a newly allocated string. Needs to
  * be freed with g_free().
-**/ 
+**/
 
 gchar *
 go_get_mime_type (gchar const *uri)
@@ -1107,7 +1112,7 @@ go_get_mime_type (gchar const *uri)
  *
  * returns: the mime type for the data as a newly allocated string. Needs to
  * be freed with g_free().
-**/ 
+**/
 gchar *
 go_get_mime_type_for_data (gconstpointer data, int data_size)
 {
@@ -1147,12 +1152,11 @@ go_get_mime_type_for_data (gconstpointer data, int data_size)
  * returns: the description for the mime type as a newly allocated string.
  * Needs to be freed with g_free(). If the description is not found, the
  * mime type itself will be returned.
-**/ 
+**/
 
 gchar *
 go_mime_type_get_description (gchar const *mime_type)
 {
-#ifdef HAVE_G_CONTENT_TYPE_FROM_MIME_TYPE
 	char *content_type = g_content_type_from_mime_type (mime_type);
 	char *description = NULL;
 	if (content_type) {
@@ -1160,9 +1164,6 @@ go_mime_type_get_description (gchar const *mime_type)
 		g_free (content_type);
 	}
 	return description ? description : g_strdup (mime_type);
-#else
-	return g_content_type_get_description (mime_type);
-#endif
 }
 
 /* ------------------------------------------------------------------------- */
